@@ -68,11 +68,13 @@ gpl_read(void)
 		wpt_tmp->longitude = le_read_double(&gp.lon);
 		alt_feet = le_read_double(&gp.alt);
 		wpt_tmp->altitude = FEET_TO_METERS(alt_feet);
+		if (wpt_tmp->altitude <= unknown_alt + 1)
+			wpt_tmp->altitude = unknown_alt;
 		wpt_tmp->creation_time = le_read32(&gp.tm);
 		
-	        wpt_tmp->course = le_read_double(&gp.heading);
-		wpt_tmp->speed = le_read_double(&gp.speed);	
-	        wpt_tmp->speed = MILES_TO_METERS(wpt_tmp->speed)/3600;	
+	        WAYPT_SET(wpt_tmp, course, le_read_double(&gp.heading));
+		WAYPT_SET(wpt_tmp, speed, le_read_double(&gp.speed));
+	        WAYPT_SET(wpt_tmp, speed, MILES_TO_METERS(wpt_tmp->speed)/3600);	
 		
 		track_add_wpt(track_head, wpt_tmp);
 	}
